'''title'''
__title__ = 'musicdl'
'''description'''
__description__ = 'A lightweight music downloader written by pure python'
'''url'''
__url__ = 'https://github.com/CharlesPikachu/musicdl'
'''version'''
__version__ = '2.2.4'
'''author'''
__author__ = 'Charles'
'''email'''
__email__ = 'charlesblwx@gmail.com'
'''license'''
__license__ = 'MIT License'
'''copyright'''
__copyright__ = 'Copyright 2020 Charles'